package solutions.bitbadger.documents.kotlinx.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("KotlinX | SQLite: Json")
class SQLiteJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    fun allDefault() =
        SQLiteDB().use(JsonFunctions::allDefault)

    @Test
    @DisplayName("all succeeds with an empty table")
    fun allEmpty() =
        SQLiteDB().use(JsonFunctions::allEmpty)

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    fun byIdString() =
        SQLiteDB().use(JsonFunctions::byIdString)

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    fun byIdNumber() =
        SQLiteDB().use(JsonFunctions::byIdNumber)

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    fun byIdNotFound() =
        SQLiteDB().use(JsonFunctions::byIdNotFound)

    @Test
    @DisplayName("byFields retrieves matching documents")
    fun byFieldsMatch() =
        SQLiteDB().use(JsonFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    fun byFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchOrdered)

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    fun byFieldsMatchNumIn() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchNumIn)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    fun byFieldsMatchInArray() =
        SQLiteDB().use(JsonFunctions::byFieldsMatchInArray)

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    fun byFieldsNoMatchInArray() =
        SQLiteDB().use(JsonFunctions::byFieldsNoMatchInArray)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::byJsonPathMatch) }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    fun firstByFieldsMatchOne() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchOne)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    fun firstByFieldsMatchMany() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchMany)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    fun firstByFieldsMatchOrdered() =
        SQLiteDB().use(JsonFunctions::firstByFieldsMatchOrdered)

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    fun firstByFieldsNoMatch() =
        SQLiteDB().use(JsonFunctions::firstByFieldsNoMatch)

    @Test
    @DisplayName("firstByContains fails")
    fun firstByContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::firstByContainsMatchOne) }
    }

    @Test
    @DisplayName("firstByJsonPath fails")
    fun firstByJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(JsonFunctions::firstByJsonPathMatchOne) }
    }
}